package jp.riken.common.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public abstract class DialogBase extends JDialog {

    private static final long serialVersionUID = 1L;

    public DialogBase(Frame owner, String title, boolean modal) {
        super(owner, title, modal);
        createComponent();
        setCenterTo(owner);
    }

    protected abstract void createComponent();

    public void setCenterTo(Component component) {
        Point ownerLocation = component.getLocation();
        Dimension ownerSize = component.getSize();
        Dimension size = getSize();
        setLocation(ownerLocation.x + (ownerSize.width - size.width) / 2,
                ownerLocation.y + (ownerSize.height - size.height) / 2);
    }

    protected boolean hasValue(JTextField textFiled) {
        return textFiled.getText().trim().length() > 0;
    }

    protected int getAsInt(JTextField textFiled, int defaultValue) {
        try {
            return Integer.parseInt(textFiled.getText().trim());
        } catch (NumberFormatException e) {
            return defaultValue;
        }
    }

    protected float getAsFloat(JTextField textFiled, float defaultValue) {
        try {
            return Float.parseFloat(textFiled.getText().trim());
        } catch (NumberFormatException e) {
            return defaultValue;
        }
    }

    protected boolean isFloat(JTextField textFiled) {
        try {
            Float.parseFloat(textFiled.getText().trim());
            return true;
        } catch (NumberFormatException e) {
            return false;
        }
    }

    protected JPanel createPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
        return panel;
    }

    protected JLabel createLabel(String title, Font font) {
        JLabel label = new JLabel(title);
        label.setFont(font);
        label.setMaximumSize(label.getPreferredSize());
        return label;
    }

    protected JTextField createTextField(Font font, DocumentListener listener) {
        return createTextField(font, listener, 0);
    }

    protected JTextField createTextField(Font font, DocumentListener listener,
            int w) {
        JTextField textField = new JTextField();
        textField.setFont(font);
        int h = textField.getPreferredSize().height;
        if (w > 0) {
            textField.setPreferredSize(new Dimension(w, h));
            textField.setMaximumSize(new Dimension(w, h));
        } else {
            textField.setMaximumSize(new Dimension(Short.MAX_VALUE, h));
        }
        if (listener != null) {
            textField.getDocument().addDocumentListener(listener);
        }
        return textField;
    }

    protected JButton createButton(String title, Font font,
            ActionListener listener) {
        return createButton(title, font, listener, null);
    }

    protected JButton createButton(String title, Font font,
            ActionListener listener, Dimension maxSize) {
        JButton button = new JButton(title);
        button.setFont(font);
        if (maxSize != null) {
            button.setMaximumSize(maxSize);
        } else {
            button.setMaximumSize(button.getPreferredSize());
        }
        button.addActionListener(listener);
        return button;
    }
}
