package jp.riken.plot2d.data;

import java.io.*;
import java.util.*;
import jp.riken.common.exception.*;

public class DataFileReaderPca extends DataFileReader {

    // file format parameters
    private int NCOLUMN_PCA_LINE = 7;
    private int NDATA_PCA = 3;
    private int NCOLUMN_START_PCA = 4;
    private int NROW_START_PCA_LINE = 3;

    public PlotDataPca read(File file, float start, float step)
            throws CustomException {
        int ncolumn = 0;
        try {
            BufferedReader reader = openReader(file);
            try {
                ncolumn = countColumns(reader);
            } finally {
                closeReader(reader);
            }
            if (ncolumn != NCOLUMN_PCA_LINE) {
                throw new CustomException("Invalid file format.");
            }

            reader = openReader(file);
            try {
                PlotDataPca data = parse(reader, ncolumn, start, step);
                if (data.getNumT() == 0) {
                    throw new CustomException("no data in file.");
                }
                data.makeXpmData();
                return data;
            } finally {
                closeReader(reader);
            }

        } catch (IOException e) {
            throw new CustomException(e);
        }
    }

    private int countColumns(BufferedReader reader) throws IOException {
        int ncolumn = 0;
        String s = null;
        int nline = 1;
        while ((s = reader.readLine()) != null) {
            if (nline == NROW_START_PCA_LINE) {
                StringTokenizer st = new StringTokenizer(s);
                ncolumn = st.countTokens();
                break;
            }
            nline++;
        }
        return ncolumn;
    }

    private PlotDataPca parse(BufferedReader reader, int ncolumn, float start,
            float step) throws IOException {
        PlotDataPca data = new PlotDataPca();

        int nrow = -999;
        int nline = 1;
        String s = null;
        while ((s = reader.readLine()) != null) {
            StringTokenizer st = new StringTokenizer(s);
            if (nline == NROW_START_PCA_LINE - 1) {
                if (st.countTokens() != 1) {
                    throw new CustomException("Invalid file format.");
                }
                nrow = Integer.parseInt(st.nextToken());
                data.setup(nrow);
            }
            if (nline >= NROW_START_PCA_LINE) {
                if (st.countTokens() != 7) {
                    throw new CustomException("Invalid file format.");
                }
                float[] values = new float[NDATA_PCA];
                float time = 0.0f;
                try {
                    int j = 0;
                    for (int i = 0; i < ncolumn; i++) {
                        String s1 = st.nextToken();
                        if (i == NCOLUMN_START_PCA - 1) {
                            time = start + step * (Integer.parseInt(s1) - 1);
                        } else if (i >= NCOLUMN_START_PCA) {
                            values[j] = Float.parseFloat(s1);
                            j++;
                        }
                    }
                } catch (NumberFormatException e) {
                    throw new CustomException("Invalid file format.");
                }
                data.setPcaValue((nline - 3), time, values);
            }
            nline++;
            if (nline == nrow + 3) {
                break;
            }
        }
        return data;
    }
}
