package jp.riken.plot2d.data;

import java.awt.*;
import java.io.*;
import java.util.*;
import jp.riken.common.exception.*;

public class DataFileReaderXpm extends DataFileReader {

    public PlotDataXpm read(File file) throws CustomException {
        try {
            BufferedReader reader = openReader(file);
            try {
                PlotDataXpm data = parse(reader);
                if (data.getNumX() == 0) {
                    throw new CustomException("Invalid file format.");
                }
                data.createLabelGrid();
                data.parseLabelX();
                return data;
            } finally {
                closeReader(reader);
            }

        } catch (IOException e) {
            throw new CustomException(e);
        }
    }

    private PlotDataXpm parse(BufferedReader reader) throws IOException {
        PlotDataXpm data = new PlotDataXpm();
        boolean hitSize = false;
        boolean hitAxis = false;
        int nx = 0;
        int ny = 0;
        int nxc = 0;
        int nyc = 0;
        int iy = 0;
        String s = null;
        while ((s = reader.readLine()) != null) {
            StringTokenizer st = new StringTokenizer(s);
            if (!hitSize && s.startsWith("/*")) {
                st.nextToken();
                String s1 = st.nextToken();
                if ("title:".equals(s1)) {
                    data.setTitle(getQuotedValue(s, ""));
                } else if ("subtitle:".equals(s1)) {
                    data.setSubtitle(getQuotedValue(s, ""));
                } else if ("legend:".equals(s1)) {
                    data.setLegend(getQuotedValue(s, ""));
                } else if ("x-label:".equals(s1)) {
                    data.setLabelX(getQuotedValue(s, ""));
                } else if ("y-label:".equals(s1)) {
                    data.setLabelY(getQuotedValue(s, ""));
                }
            } else if (!hitSize && s.startsWith("\"")) {
                String[] strAry = s.split("\"");
                if (strAry.length > 1) {
                    StringTokenizer st2 = new StringTokenizer(strAry[1]);
                    try {
                        nx = Integer.parseInt(st2.nextToken());
                        ny = Integer.parseInt(st2.nextToken());
                        if (nx <= 0 || ny <= 0) {
                            throw new CustomException("Invalid file format.");
                        }
                        data.setup(nx, ny);
                    } catch (NumberFormatException e) {
                        throw new CustomException("Invalid file format.");
                    }
                    hitSize = true;
                }
            } else if (hitSize && !hitAxis && s.startsWith("\"")) {
                String[] strAry = s.split("\"");
                if (strAry.length > 1) {
                    char element = strAry[1].charAt(0);
                    String colorString = strAry[1].substring(6, 12);
                    Color color = new Color(Integer.parseInt(colorString, 16));
                    String legend = strAry[3];
                    data.addPointValue(new PointValue(element, color, legend));
                }
            } else if (hitSize && s.startsWith("/*")) {
                st.nextToken();
                String s1 = st.nextToken();
                try {
                    if ("x-axis:".equals(s1)) {
                        for (; nxc < nx; nxc++) {
                            String s2 = st.nextToken();
                            if (s2.equals("*/")) {
                                break;
                            }
                            data.setXaxis(nxc, Integer.parseInt(s2));
                        }
                    } else if ("y-axis:".equals(s1)) {
                        for (; nyc < ny; nyc++) {
                            String s2 = st.nextToken();
                            if (s2.equals("*/")) {
                                break;
                            }
                            data.setYaxis(nyc, Integer.parseInt(s2));
                        }
                        hitAxis = true;
                    }
                } catch (NumberFormatException e) {
                    throw new CustomException("Invalid file format.");
                }
            } else if (hitSize && hitAxis && s.startsWith("\"")) {
                String[] strAry = s.split("\"");
                if (strAry.length > 1) {
                    if (strAry[1].length() != nx) {
                        throw new CustomException("Invalid file format.");
                    }
                    for (int ix = 0; ix < nx; ix++) {
                        data.setValue(ix, ny-iy-1, strAry[1].charAt(ix));
                    }
                    iy++;
                }
            }
        }
        return data;
    }
}
