package jp.riken.plot2d.data;

import java.io.*;
import java.util.*;
import jp.riken.common.exception.*;

public class DataFileReaderXvg extends DataFileReader {

    public PlotDataXvg read(File file) throws CustomException {
        int ncolumn = 0;
        try {
            BufferedReader reader = openReader(file);
            try {
                ncolumn = countColumns(reader);                
            } finally {
                closeReader(reader);
            }
            if (ncolumn < 2) {
                throw new CustomException("Invalid file format.");
            }

            reader = openReader(file);
            try {
                PlotDataXvg data = parse(reader, ncolumn);
                if (data.getNumX() == 0) {
                    throw new CustomException("Invalid file format.");
                }
                data.createLabelGrid();
                data.parseLabelX();
                return data;
            } finally {
                closeReader(reader);
            }

        } catch (IOException e) {
            throw new CustomException(e);
        }
    }

    private int countColumns(BufferedReader reader) throws IOException {
        int ncolumn = 0;
        String s = null;
        while ((s = reader.readLine()) != null) {
            if (s.startsWith("#") || s.startsWith("@")) {
                continue;
            }
            if (s.startsWith("/") || s.startsWith("\"")) {
                // XPM file probably
                throw new CustomException("Invalid file format.");
            }
            StringTokenizer st = new StringTokenizer(s);
            ncolumn = st.countTokens();
            break;
        }
        return ncolumn;
    }

    private PlotDataXvg parse(BufferedReader reader, int ncolumn)
            throws IOException {
        int numY = ncolumn - 1;
        PlotDataXvg data = new PlotDataXvg();
        data.setup(numY);
        String s = null;
        while ((s = reader.readLine()) != null) {
            if (s.startsWith("#")) {
                continue;
            }
            if (s.startsWith("/") || s.startsWith("\"")) {
                // XPM file probably
                throw new CustomException("Invalid file format.");
            }
            StringTokenizer st = new StringTokenizer(s);
            if (s.startsWith("@")) {
                st.nextToken();
                String s1 = st.nextToken();
                if ("title".equals(s1)) {
                    data.setTitle(getQuotedValue(s, ""));
                } else if ("subtitle".equals(s1)) {
                    data.setSubtitle(getQuotedValue(s, ""));
                } else if ("xaxis".equals(s1)) {
                    String s2 = st.nextToken();
                    if ("label".equals(s2)) {
                        data.setLabelX(getQuotedValue(s, ""));
                    }
                } else if ("yaxis".equals(s1)) {
                    String s2 = st.nextToken();
                    if ("label".equals(s2)) {
                        data.setLabelY(getQuotedValue(s, ""));
                    }
                } else if (s1.startsWith("s")) {
                    int index = toInt(s1.substring(1), -1);
                    if (index >= 0 && index < numY) {
                        data.setLegend(index, getQuotedValue(s, ""));
                    }
                }
            } else {
                if (st.countTokens() != ncolumn) {
                    throw new CustomException("Invalid file format.");
                }
                float[] values = new float[ncolumn];
                try {
                    for (int i = 0; i < ncolumn; i++) {
                        values[i] = Float.parseFloat(st.nextToken());
                    }
                } catch (NumberFormatException e) {
                    throw new CustomException("Invalid file format.");
                }
                data.addPoints(values);
            }
        }
        return data;
    }
}
