package jp.riken.plot2d.data;

public abstract class PlotData {

    protected String title;
    protected String subtitle;
    protected String labelX;
    protected String labelY;
    protected Grid labelGrid;
    protected int numX;
    protected int numY;

    protected Float pickValueX;
    protected Integer pickIndexX;

    protected boolean timeAware;
    protected boolean xaxisTime;
    protected boolean pickAware;
    protected float xaxisScale;

    public PlotData() {
        title = "";
        subtitle = "";
        labelX = "";
        labelY = "";
        clearPick();
        timeAware = true;
        xaxisTime = true;
        pickAware = true;
        xaxisScale = 1;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getSubtitle() {
        return subtitle;
    }

    public void setSubtitle(String subtitle) {
        this.subtitle = subtitle;
    }

    public String getLabelX() {
        return labelX;
    }

    public void setLabelX(String labelX) {
        this.labelX = labelX;
    }

    public String getLabelY() {
        return labelY;
    }

    public void setLabelY(String labelY) {
        this.labelY = labelY;
    }

    public Grid getLabelGrid() {
        return labelGrid;
    }

    public int getNumX() {
        return numX;
    }

    public int getNumY() {
        return numY;
    }

    public Float getPickValueX() {
        return pickValueX;
    }

    public void setPickValueX(Float pickValueX) {
        this.pickValueX = pickValueX;
    }

    public Integer getPickIndexX() {
        return pickIndexX;
    }

    public void setPickIndexX(Integer pickIndexX) {
        this.pickIndexX = pickIndexX;
    }

    public boolean isTimeAware() {
        return timeAware;
    }

    public boolean isXaxisTime() {
        return xaxisTime;
    }

    public boolean isPickAware() {
        return pickAware;
    }

    public float getXaxisScale() {
        return xaxisScale;
    }

    public void clearPick() {
        pickValueX = null;
        pickIndexX = null;
    }

    public void parseLabelX() {
        if (labelX.startsWith("Time ")) {
            int idx1 = labelX.indexOf("(");
            int idx2 = labelX.indexOf(")");
            String unit = labelX.substring(idx1 + 1, idx2);
            if (unit.equals("ns")) {
                xaxisScale = 1000;
            } else {
                xaxisScale = 1; // assume ps
            }
        } else {
            timeAware = false;
            xaxisTime = false;
        }
    }

    public abstract float getX(int ix);

    public abstract void createLabelGrid();

    protected void refineLabelGrid() {
        double minX = labelGrid.getMinX();
        double maxX = labelGrid.getMaxX();
        double minY = labelGrid.getMinY();
        double maxY = labelGrid.getMaxY();
        double widthX = labelGrid.getWidthX();
        double widthY = labelGrid.getWidthY();
        int countX = labelGrid.getCountX();
        int countY = labelGrid.getCountY();

        // refinement x y scale
        double ketaX = Math.floor(Math.log10(widthX));
        double ketaY = Math.floor(Math.log10(widthY));
        double unitX = Math.pow(10.0, ketaX);
        double unitY = Math.pow(10.0, ketaY);
        widthX = Math.ceil(widthX / unitX) * unitX;
        widthY = Math.ceil(widthY / unitY) * unitY;
        minX = Math.floor(minX / unitX) * unitX;
        minY = Math.floor(minY / unitY) * unitY;
        
        // refinement countX countY
        for(int n = 0; n < countX+10; n++) {
            if((minX + n * widthX) >= maxX) {
                countX = n;
                break;
            }
        }
        for(int n = 0; n < countY+10; n++) {
            if((minY + n * widthY) >= maxY) {
                countY = n;
                break;
            }
        }

        labelGrid = new Grid();
        labelGrid.setMinX(minX);
        labelGrid.setMinY(minY);
        labelGrid.setMaxX(minX + widthX * countX);
        labelGrid.setMaxY(minY + widthY * countY);
        labelGrid.setWidthX(widthX);
        labelGrid.setWidthY(widthY);
        labelGrid.setCountX(countX);
        labelGrid.setCountY(countY);
    }

    public void setTime(float time) {
        time /= xaxisScale;
        pickValueX = time;
        int ix = findNearestX(time);
        if (getX(ix) == time) {
            pickIndexX = ix;
        }
    }

    public void picked(float xc) {
        clearPick();
        int ix = findNearestX(xc);
        if (ix >= 0) {
            pickIndexX = ix;
            pickValueX = getX(ix);
        }
    }

    public int findNearestX(float xc) {
        if (numX == 0) {
            return -1;
        }
        int l = 0;
        int h = numX - 1;
        if (xc < getX(l)) {
            return l;
        }
        if (xc > getX(h)) {
            return h;
        }
        // bisection
        while (h - l > 1) {
            int m = (h + l) / 2;
            if (xc < getX(m)) {
                h = m;
            } else {
                l = m;
            }
        }
        float dl = xc - getX(l);
        float dh = getX(h) - xc;
        return (dl <= dh) ? l : h;
    }
}
