package jp.riken.plot2d.data;

public class PlotDataPca extends PlotData {

    private int numT;
    private float[] times;
    private float[] pca1;
    private float[] pca2;
    private float[] pca3;
    private PlotDataPcaEach[] pcas;

    public PlotDataPca() {
        xaxisTime = false;
        pickAware = false;
        pcas = new PlotDataPcaEach[4];
        for (int i = 0; i < 4; i++) {
            pcas[i] = new PlotDataPcaEach();
        }
    }

    @Override
    public float getX(int ix) {
        return 0;
    }

    @Override
    public void createLabelGrid() {
    }

    public void setup(int numT) {
        this.numT = numT;
        times = new float[numT];
        pca1 = new float[numT];
        pca2 = new float[numT];
        pca3 = new float[numT];
        pcas[0].setup(numT, pca1, pca2);
        pcas[1].setup(numT, pca3, pca2);
        pcas[2].setup(numT, pca1, pca3);
        pcas[3].setup(numT, pca1, pca1);
    }

    public void setPcaValue(int index, float time, float[] values) {
        times[index] = time;
        pca1[index] = values[0];
        pca2[index] = values[1];
        pca3[index] = values[2];
    }

    public int getNumT() {
        return this.numT;
    }

    public PlotDataPcaEach[] getPcas() {
        return pcas;
    }

    public void makeXpmData() {
        float min1 = Float.POSITIVE_INFINITY;
        float max1 = Float.NEGATIVE_INFINITY;
        float min2 = Float.POSITIVE_INFINITY;
        float max2 = Float.NEGATIVE_INFINITY;
        float min3 = Float.POSITIVE_INFINITY;
        float max3 = Float.NEGATIVE_INFINITY;
        for (int i = 0; i < numT; i++) {
            if (min1 > pca1[i]) {
                min1 = pca1[i];
            }
            if (max1 < pca1[i]) {
                max1 = pca1[i];
            }
            if (min2 > pca2[i]) {
                min2 = pca2[i];
            }
            if (max2 < pca2[i]) {
                max2 = pca2[i];
            }
            if (min3 > pca3[i]) {
                min3 = pca3[i];
            }
            if (max3 < pca3[i]) {
                max3 = pca3[i];
            }
        }
        pcas[0].makeXpmData("PCA", "PC1", "PC2", min1, max1, min2, max2);
        pcas[1].makeXpmData("",    "PC3", "PC2", min3, max3, min2, max2);
        pcas[2].makeXpmData("",    "PC1", "PC3", min1, max1, min3, max3);
        pcas[3].makeXpmData("",    "PC1", "PC1", min1, max1, min1, max1);
    }

    @Override
    public void setTime(float time) {
        int it = findNearestT(time);
        if (it < 0 || times[it] != time) {
            it = -1;
        }
        for (PlotDataPcaEach pca : pcas) {
            pca.setTimeIndex(it);
        }
    }

    private int findNearestT(float time) {
        if (numT == 0) {
            return -1;
        }
        int l = 0;
        int h = numT - 1;
        if (time < times[l]) {
            return l;
        }
        if (time > times[h]) {
            return h;
        }
        // bisection
        while (h - l > 1) {
            int m = (h + l) / 2;
            if (time < times[m]) {
                h = m;
            } else {
                l = m;
            }
        }
        float dl = time - times[l];
        float dh = times[h] - time;
        return (dl <= dh) ? l : h;
    }
}
