package jp.riken.plot2d.data;

import java.awt.Color;

public class PlotDataPcaEach extends PlotDataXpm {

    private static final int NLEGEND = 7;
    private int numT;
    private float[] pcaX;
    private float[] pcaY;
    private int[] tx;
    private int[] ty;
    private Integer pickIndexY;

    public void setup(int numT, float[] pcaX, float[] pcaY) {
        this.numT = numT;
        this.pcaX = pcaX;
        this.pcaY = pcaY;
        tx = new int[numT];
        ty = new int[numT];
    }

    public void makeXpmData(String title, String labelX, String labelY,
            float minX, float maxX, float minY, float maxY) {
        this.title = title;
        this.labelX = labelX;
        this.labelY = labelY;

        // create grid
        int div = 51;
        setup(div, div);
        float widthX = (maxX - minX) / (div - 1);
        float widthY = (maxY - minY) / (div - 1);
        for (int i = 0; i< div; i++) {
            setXaxis(i, minX + i * widthX);
            setYaxis(i, minY + i * widthY);
        }
        createLabelGrid();
        
        // reallocate
        int cx = labelGrid.getCountX();
        int cy = labelGrid.getCountY();
        setup(cx * 10, cy * 10);

        // count
        int[][] counts = new int[numX][numY];
        for (int ix = 0; ix < numX; ix++) {
            for (int iy = 0; iy < numY; iy++) {
                counts[ix][iy] = 0;
            }
        }
        double maxx = labelGrid.getMaxX();
        double maxy = labelGrid.getMaxY();
        double minx = labelGrid.getMinX();
        double miny = labelGrid.getMinY();
        for (int i = 0; i < numT; i++) {
            int ix = getI(minx, maxx, numX, pcaX[i]);
            int iy = getI(miny, maxy, numY, pcaY[i]);
            counts[ix][iy]++;
            tx[i] = ix;
            ty[i] = iy;
        }

        // get range of legend
        int ncMax = 0;
        for (int ix = 0; ix < numX; ix++) {
            for (int iy = 0; iy < numY; iy++) {
                if (ncMax < counts[ix][iy]) {
                    ncMax = counts[ix][iy];
                }
            }
        }
        double wnc = ncMax / (double) NLEGEND;
        boolean ncSmall = (wnc <= 1.0);

        // make legend
        if (ncSmall) {
            addPointValue(new PointValue('0', new Color(0x000000), "0"));
            addPointValue(new PointValue('1', new Color(0x4169E1), "1"));
            addPointValue(new PointValue('2', new Color(0x1E90FF), "2"));
            addPointValue(new PointValue('3', new Color(0x7B68EE), "3"));
            addPointValue(new PointValue('4', new Color(0xBA55D3), "4"));
            addPointValue(new PointValue('5', new Color(0xC71585), "5"));
            addPointValue(new PointValue('6', new Color(0xDA70D6), "6"));
            addPointValue(new PointValue('7', new Color(0xFFFF00), "7"));
        } else {
            addPointValue(new PointValue('0', new Color(0x000000), "0"));
            addPointValue(new PointValue('1', new Color(0x4169E1),
                    "1 - " + (int) Math.floor(wnc * 1)));
            addPointValue(new PointValue('2', new Color(0x1E90FF),
                    (int) Math.ceil(wnc * 1) + " - " + (int) Math.floor(wnc * 2)));
            addPointValue(new PointValue('3', new Color(0x7B68EE),
                    (int) Math.ceil(wnc * 2) + " - " + (int) Math.floor(wnc * 3)));
            addPointValue(new PointValue('4', new Color(0xBA55D3),
                    (int) Math.ceil(wnc * 3) + " - " + (int) Math.floor(wnc * 4)));
            addPointValue(new PointValue('5', new Color(0xC71585),
                    (int) Math.ceil(wnc * 4) + " - " + (int) Math.floor(wnc * 5)));
            addPointValue(new PointValue('6', new Color(0xDA70D6),
                    (int) Math.ceil(wnc * 5) + " - " + (int) Math.floor(wnc * 6)));
            addPointValue(new PointValue('7', new Color(0xFFFF00),
                    (int) Math.ceil(wnc * 6) + " - " + ncMax));
        }

        // make XPM data
        if (ncSmall) {
            for (int ix = 0; ix < numX; ix++) {
                for (int iy = 0; iy < numY; iy++) {
                    int nc = counts[ix][iy];
                    if (nc == 0) {
                        setValue(ix, iy, '0');
                    } else if (nc == 1) {
                        setValue(ix, iy, '1');
                    } else if (nc == 2) {
                        setValue(ix, iy, '2');
                    } else if (nc == 3) {
                        setValue(ix, iy, '3');
                    } else if (nc == 4) {
                        setValue(ix, iy, '4');
                    } else if (nc == 5) {
                        setValue(ix, iy, '5');
                    } else if (nc == 6) {
                        setValue(ix, iy, '6');
                    } else if (nc == 7) {
                        setValue(ix, iy, '7');
                    }
                }
            }
        } else {
            for (int ix = 0; ix < numX; ix++) {
                for (int iy = 0; iy < numY; iy++) {
                    double nc = (double) counts[ix][iy];
                    if (nc == 0) {
                        setValue(ix, iy, '0');
                    } else if (nc <= wnc * 1) {
                        setValue(ix, iy, '1');
                    } else if (nc <= wnc * 2) {
                        setValue(ix, iy, '2');
                    } else if (nc <= wnc * 3) {
                        setValue(ix, iy, '3');
                    } else if (nc <= wnc * 4) {
                        setValue(ix, iy, '4');
                    } else if (nc <= wnc * 5) {
                        setValue(ix, iy, '5');
                    } else if (nc <= wnc * 6) {
                        setValue(ix, iy, '6');
                    } else {
                        setValue(ix, iy, '7');
                    }
                }
            }
        }
    }

    private int getI(double min, double max, int n, float pca) {
        int in = 0;
        double delta = (max - min) / n;
        for (int i = 0; i < n; i++) {
            if (pca >= (delta * i + min) && pca < (delta * (i + 1) + min)) {
                in = i;
                break;
            }
        }
        return in;
    }

    public void setTimeIndex(int it) {
        if (it >= 0) {
            pickIndexX = tx[it];
            pickIndexY = ty[it];
        } else {
            pickIndexX = null;
            pickIndexY = null;
        }
    }

    @Override
    public boolean isHighlight(int ix, int iy) {
        return pickIndexX != null && pickIndexX == ix && pickIndexY == iy;
    }
}
