package jp.riken.plot2d.data;

import java.util.*;
import jp.riken.common.exception.*;

public class PlotDataXpm extends PlotData {

    private String legend;
    private float[] xaxis;
    private float[] yaxis;
    private int[][] indexes;
    private List<PointValue> values;
    private Map<Character, Integer> valueMap;

    public PlotDataXpm() {
        legend = "";
        values = new ArrayList<PointValue>();
        valueMap = new HashMap<Character, Integer>();
    }

    public void setup(int numX, int numY) {
        this.numX = numX;
        this.numY = numY;
        xaxis = new float[numX];
        yaxis = new float[numY];
        indexes = new int[numX][numY];
    }

    public String getLegend() {
        return legend;
    }

    public void setLegend(String legend) {
        this.legend = legend;
    }

    public void setXaxis(int index, float x) {
        xaxis[index] = x;
    }

    public float getXaxis(int index) {
        return xaxis[index];
    }

    public void setYaxis(int index, float y) {
        yaxis[index] = y;
    }

    public float getYaxis(int index) {
        return yaxis[index];
    }

    @Override
    public float getX(int ix) {
        return getXaxis(ix);
    }

    public void addPointValue(PointValue pointValue) {
        valueMap.put(pointValue.getElement(), values.size());
        values.add(pointValue);
    }

    public int numPointValues() {
        return values.size();
    }

    public PointValue getPointValue(int index) {
        return values.get(index);
    }

    public void setValue(int ix, int iy, char element) {
        if (valueMap.get(element) == null) {
            throw new CustomException("unknown element: " + element);
        }
        indexes[ix][iy] = valueMap.get(element);
    }

    public PointValue getValue(int ix, int iy) {
        return values.get(indexes[ix][iy]);
    }

    @Override
    public void createLabelGrid() {
        float minX = xaxis[0];
        float minY = yaxis[0];
        float maxX = xaxis[numX - 1];
        float maxY = yaxis[numY - 1];
        float widthX;
        float widthY;
        int countX = 1;
        int countY = 1;
        int ny = 1;
        int nx = 1;
        
        if (this.numX == 1) {
        	widthX = 1;
        	maxX = minX + 1;
        }
        else {
        	widthX = xaxis[1] - xaxis[0];
            while (true) {
                countX = (int) ((maxX - minX) / (widthX * nx));
                if (countX <= 10) {
                    break;
                }
                nx++;
            }
            widthX = (maxX - minX) / countX;
        }
        
        if (this.numY == 1) {
        	widthY = 1;
        	maxY = minY + 1;
        }
        else {
        	widthY = yaxis[1] - yaxis[0];
            while (true) {
                countY = (int) ((maxY - minY) / (widthY * ny));
                if (countY <= 10) {
                    break;
                }
                ny++;
            }
            widthY = (maxY - minY) / countY;
        }

        labelGrid = new Grid();
        labelGrid.setMinX(minX);
        labelGrid.setMinY(minY);
        labelGrid.setMaxX(maxX);
        labelGrid.setMaxY(maxY);
        labelGrid.setWidthX(widthX);
        labelGrid.setWidthY(widthY);
        labelGrid.setCountX(countX);
        labelGrid.setCountY(countY);

        refineLabelGrid();
    }

    public boolean isHighlight(int ix, int iy) {
        return false;
    }
}
