package jp.riken.plot2d.data;

import java.awt.Color;
import java.util.*;

public class PlotDataXvg extends PlotData {

    private List<float[]> points;
    private String[] legends;
    private Color[] colors;

    private float viewMinY;
    private float viewMaxY;

    public PlotDataXvg() {
        points = new ArrayList<float[]>();
    }

    public void setup(int numY) {
        this.numY = numY;
        legends = new String[numY];
        colors = new Color[numY];
        for (int i = 0; i < numY; i++) {
            legends[i] = "";
            if (i % 10 == 0) {
                colors[i] = Color.BLACK;
            } else if (i % 10 == 1) {
                colors[i] = new Color(249, 37, 0);
            } else if (i % 10 == 2) {
                colors[i] = new Color(70, 184, 109);
            } else if (i % 10 == 3) {
                colors[i] = new Color(142, 0, 204);
            } else if (i % 10 == 4) {
                colors[i] = new Color(49, 0, 178);
            } else if (i % 10 == 5) {
                colors[i] = new Color(255, 129, 25);
            } else if (i % 10 == 6) {
                colors[i] = new Color(228, 162, 11);
            } else if (i % 10 == 7) {
                colors[i] = new Color(116, 169, 214);
            } else if (i % 10 == 8) {
                colors[i] = new Color(188, 189, 194);
            } else if (i % 10 == 9) {
                colors[i] = new Color(211, 212, 155);
            }
        }
    }

    public void addPoints(float[] values) {
        points.add(values);
        numX++;
    }

    @Override
    public float getX(int ix) {
        return points.get(ix)[0];
    }

    public float getY(int ix, int iy) {
        return points.get(ix)[iy + 1];
    }

    public String getLegend(int index) {
        return legends[index];
    }

    public void setLegend(int index, String legend) {
        legends[index] = legend;
    }

    public Color getColor(int index) {
        return colors[index];
    }

    public void setColor(int index, Color color) {
        colors[index] = color;
    }

    public float getViewMinY() {
        return viewMinY;
    }

    public void setViewMinY(float viewMinY) {
        this.viewMinY = viewMinY;
    }

    public float getViewMaxY() {
        return viewMaxY;
    }

    public void setViewMaxY(float viewMaxY) {
        this.viewMaxY = viewMaxY;
    }

    @Override
    public void createLabelGrid() {
        double maxX = Double.NEGATIVE_INFINITY;
        double minX = Double.POSITIVE_INFINITY;
        double maxY = Double.NEGATIVE_INFINITY;
        double minY = Double.POSITIVE_INFINITY;
        for (int ix = 0; ix < numX; ix++) {
            float[] values = points.get(ix);
            double x = values[0];
            if (x > maxX) {
                maxX = x;
            }
            if (x < minX) {
                minX = x;
            }
            for (int iy = 0; iy < numY; iy++) {
                double y = values[iy + 1];
                if (y > maxY) {
                    maxY = y;
                }
                if (y < minY) {
                    minY = y;
                }
            }
        }

        // in case of all same values
        if (minX == maxX) {
            minX -= 1.0;
            maxX += 1.0;
        }
        if (minY == maxY) {
            minY -= 1.0;
            maxY += 1.0;
        }

        int countX = 10;
        int countY = 10;
        double widthX = (maxX - minX) / countX;
        double widthY = (maxY - minY) / countY;

        labelGrid = new Grid();
        labelGrid.setMinX(minX);
        labelGrid.setMinY(minY);
        labelGrid.setMaxX(maxX);
        labelGrid.setMaxY(maxY);
        labelGrid.setWidthX(widthX);
        labelGrid.setWidthY(widthY);
        labelGrid.setCountX(countX);
        labelGrid.setCountY(countY);

        refineLabelGrid();

        viewMinY = (float) labelGrid.getMinY();
        viewMaxY = (float) labelGrid.getMaxY();
    }
    
    public void recreateYlabelGrid(double minY, double maxY) {
    	int countY = 10;
        double widthY = (maxY - minY) / countY;
        labelGrid.setMinY(minY);
        labelGrid.setMaxY(maxY);
        labelGrid.setWidthY(widthY);
        labelGrid.setCountY(countY);
        viewMinY = (float) labelGrid.getMinY();
        viewMaxY = (float) labelGrid.getMaxY();
    }
}
