package jp.riken.plot2d.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import jp.riken.common.gui.*;

public class YaxisRangeDialog extends DialogBase implements ActionListener,
        DocumentListener {

    private static final long serialVersionUID = 1L;

    private JTextField minText;
    private JTextField maxText;
    private JButton buttonOK;
    private JButton buttonCancel;
    private JButton buttonReset;
    private boolean ok;
    private boolean reset;

    public YaxisRangeDialog(Frame owner, String title, boolean modal) {
        super(owner, title, modal);
        setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
    }

    public void initialize() {
        ok = false;
        reset = false;
    }

    public float getMin() {
        return Float.parseFloat(minText.getText());
    }

    public void setMin(float min) {
        minText.setText("" + min);
    }

    public float getMax() {
        return Float.parseFloat(maxText.getText());
    }

    public void setMax(float max) {
        maxText.setText("" + max);
    }

    public boolean isOk() {
        return ok;
    }
    
    public boolean isReset() {
        return reset;
    }

    private void toggleOkButton() {
        if (buttonOK != null) {
            buttonOK.setEnabled(validateInput());
        }
    }

    private boolean validateInput() {
        if (!hasValue(minText) || !isFloat(minText)) {
            return false;
        }
        if (!hasValue(maxText) || !isFloat(maxText)) {
            return false;
        }
        return true;
    }

    // --------------------------------------------------------------------
    // ActionListener
    // --------------------------------------------------------------------
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == buttonOK) {
            ok = true;
            reset = false;
            setVisible(false);
        } else if (e.getSource() == buttonCancel) {
            ok = false;
            reset =false;
            setVisible(false);
        } else if (e.getSource() == buttonReset) {
            ok = false;
            reset = true;
            setVisible(false);
        }
    }

    // --------------------------------------------------------------------
    // DocumentListener
    // --------------------------------------------------------------------
    public void changedUpdate(DocumentEvent e) {
    }

    public void insertUpdate(DocumentEvent e) {
        toggleOkButton();
    }

    public void removeUpdate(DocumentEvent e) {
        toggleOkButton();
    }

    // --------------------------------------------------------------------
    // Component
    // --------------------------------------------------------------------
    @Override
    protected void createComponent() {
        setSize(400, 180);
        setResizable(true);
        Container container = this.getContentPane();
        container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));

        Font font = new Font("SansSerif", Font.PLAIN, 12);
        minText = createTextField(font, this, 90);
        maxText = createTextField(font, this, 90);
        buttonCancel = createButton("Cancel", font, this);
        buttonOK = createButton("  OK  ", font, this,
                buttonCancel.getPreferredSize());
        buttonOK.setEnabled(false);
        buttonReset = createButton("Reset", font, this,
                buttonCancel.getPreferredSize());
        buttonReset.setEnabled(true);

        Font font2 = new Font("SansSerif", Font.PLAIN, 12);
        JLabel label1 = createLabel("Y-axis range: ", font2);
        JLabel label11 = createLabel("min ", font2);
        JLabel label12 = createLabel("max ", font2);

        JPanel panel = null;
        int mh1 = 20;
        int mh2 = 40;

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(1, 20)));
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(mh1, 1)));
        panel.add(label1);
        panel.add(Box.createHorizontalGlue());
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(1, 10)));
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(mh2, 1)));
        panel.add(label11);
        panel.add(minText);
        panel.add(Box.createRigidArea(new Dimension(20, 1)));
        panel.add(label12);
        panel.add(maxText);
        panel.add(Box.createHorizontalGlue());
        container.add(panel);

        container.add(Box.createVerticalGlue());

        panel = createPanel();
        panel.add(Box.createHorizontalGlue());
        panel.add(buttonOK);
        panel.add(Box.createRigidArea(new Dimension(5, 1)));
        panel.add(buttonCancel);
        panel.add(Box.createRigidArea(new Dimension(5, 1)));
        panel.add(buttonReset);
        panel.add(Box.createRigidArea(new Dimension(10, 1)));
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(1, 10)));
        container.add(panel);
    }
}
