package jp.riken.plot2d.renderer;

import java.awt.*;
import jp.riken.plot2d.data.*;

public class ImageRendererLine extends ImageRenderer {

    public ImageRendererLine(RenderingContext context) {
        super(context);
    }

    // --------------------------------------------------------------------
    // render
    // --------------------------------------------------------------------
    @Override
    protected void render(PlotData data, Graphics2D g) {
        // background
        renderBackground(g);

        if (data == null) {
            return;
        }

        readScale(data.getLabelGrid());

        PlotDataXvg dataXvg = (PlotDataXvg) data;

        // title
        renderTitle(data, g);

        // label
        renderLabel(data, g, false);

        // line
        renderLine(dataXvg, g);
        
        // legend
        renderLegend(dataXvg, g);

        // frame
        renderFrame(g);
        
        // picked line
        renderPickedLine(data, g);
        
        // picked value
        renderPickedValue(dataXvg, g);
    }

    // --------------------------------------------------------------------
    // line
    // --------------------------------------------------------------------
    private void renderLine(PlotDataXvg data, Graphics2D g) {
        for (int iy = 0; iy < data.getNumY(); iy++) {
            int xp0 = 0;
            int yp0 = 0;
            for (int ix = 0; ix < data.getNumX(); ix++) {
                double x = data.getX(ix);
                double y = data.getY(ix, iy);
                if (y < data.getLabelGrid().getMinY()) {
                	y = data.getLabelGrid().getMinY();
                }
                if (y > data.getLabelGrid().getMaxY()) {
                	y = data.getLabelGrid().getMaxY();
                }
                int xp = (int) ((x - minX) * sx + x1);
                int yp = (int) (h - (y - minY) * sy - y1);
                g.setColor(data.getColor(iy));
                if (ix != 0) {
                    g.drawLine(xp, yp, xp0, yp0);
                }
                xp0 = xp;
                yp0 = yp;
            }
        }
    }

    // --------------------------------------------------------------------
    // legend
    // --------------------------------------------------------------------
    private void renderLegend(PlotDataXvg data, Graphics2D g) {
        int fontSize1 = (int) (14 * ratio);
        Font font1 = new Font("Dialog", Font.PLAIN, fontSize1);
        FontMetrics fm1 = g.getFontMetrics(font1);

        for (int iy = 0; iy < data.getNumY(); iy++) {
            String legend = data.getLegend(iy);
            double y = data.getY(data.getNumX() - 1, iy);

            int xp = (int) (x2 + 0.5 * fm1.getHeight());
            int yp = (int) (h - (y - minY) * sy - y1 + 0.7 * fm1.getAscent());

            g.setColor(data.getColor(iy));
            g.setFont(font1);
            g.drawString(legend, xp, yp);
        }
    }

    // --------------------------------------------------------------------
    // picked value
    // --------------------------------------------------------------------
    private void renderPickedValue(PlotDataXvg data, Graphics2D g) {
        if (data.getPickIndexX() == null) {
            return;
        }
        int fontSize1 = (int) (14 * ratio);
        Font font1 = new Font("Dialog", Font.PLAIN, fontSize1);
        FontMetrics fm1 = g.getFontMetrics(font1);
        int ascent1 = fm1.getAscent();

        int ix = data.getPickIndexX();
        double x = data.getX(ix);

        // get total length of strings
        int ts = 0;
        for (int iy = 0; iy < data.getNumY(); iy++) {
            double y = data.getY(ix, iy);
            if (iy == 0) {
                ts = fm1.stringWidth("x = ");
                ts += 10 + fm1.stringWidth(formatValue(x));
                ts += fm1.stringWidth("y = ");
            }
            ts += 5 + fm1.stringWidth(formatValue(y));
        }
        int xp = 0;
        int yp = (int) (h - y1 + 4.5 * ascent1);
        for (int iy = 0; iy < data.getNumY(); iy++) {
            double y = data.getY(ix, iy);
            if (iy == 0) {
                xp = (int) ((x - minX) * sx + x1);
                xp = xp - (int) (ts * 0.5);
                if (xp < (int) x1) {
                    xp = (int) x1;
                } else if (xp + (int) ts > (int) x2) {
                    xp = (int) x2 - ts;
                }
                String xes = "x = ";
                g.setColor(Color.BLACK);
                g.setFont(font1);
                g.drawString(xes, xp, yp);
                xp += fm1.stringWidth(xes);

                String xs = formatValue(x);
                g.drawString(xs, xp, yp);
                xp += 10 + fm1.stringWidth(xs);

                String yes = "y = ";
                g.drawString(yes, xp, yp);
                xp += fm1.stringWidth(yes);
            }
            String ys = formatValue(y);
            g.setColor(data.getColor(iy));
            g.drawString(ys, xp, yp);
            xp += 5 + fm1.stringWidth(ys);
        }
    }
}
