package jp.riken.plot2d.renderer;

import java.awt.*;
import jp.riken.plot2d.data.*;

public class ImageRendererTile extends ImageRenderer {

    private boolean pca = false;

    public ImageRendererTile(RenderingContext context, boolean pca1) {
        super(context);
        pca = pca1;
    }

    // --------------------------------------------------------------------
    // render
    // --------------------------------------------------------------------
    @Override
    protected void render(PlotData data, Graphics2D g) {
        // background
        renderBackground(g);

        if (data == null) {
            return;
        }

        if (!pca) {
            readScale(data.getLabelGrid());

            PlotDataXpm dataXpm = (PlotDataXpm) data;

            // title
            renderTitle(dataXpm, g);

            // label
            renderLabel(data, g, pca);

            // tile
            renderTile(dataXpm, g);

            // legend
            renderLegend(dataXpm, g, pca);

            // frame
            renderFrame(g);

            // picked line
            renderPickedLine(data, g);

            // picked value
            renderPickedValue(dataXpm, g);
        } else {
            PlotDataPca dataPcaAll = (PlotDataPca) data;
            for (int i = 0; i < 4; i++) {
                PlotDataPcaEach dataPca = dataPcaAll.getPcas()[i];

                readScale(dataPca.getLabelGrid(), i);

                // title
                if (i == 0) {
                    renderTitle(dataPca, g);
                }

                // label
                renderLabel(dataPca, g, pca);

                // tile
                renderTile(dataPca, g);

                // legend
                renderLegend(dataPca, g, pca);

                // frame
                renderFrame(g);
            }
        }
    }

    // --------------------------------------------------------------------
    // tile
    // --------------------------------------------------------------------
    private void renderTile(PlotDataXpm data, Graphics2D g) {
        int w1 = (int) Math.ceil((x2 - x1) / data.getNumX());
        int h1 = (int) Math.ceil((y2 - y1) / data.getNumY());

        Color highlight = Color.WHITE;
        for (int ix = 0; ix < data.getNumX(); ix++) {
            for (int iy = 0; iy < data.getNumY(); iy++) {
                if (data.isHighlight(ix, iy)) {
                    g.setColor(highlight);
                } else {
                    g.setColor(data.getValue(ix, iy).getColor());
                }

                double ssx = (maxX - minX) / data.getNumX();
                double ssy = (maxY - minY) / data.getNumY();
                double x = minX + ix * ssx;
                double y = minY + iy * ssy;

                int xp = (int) ((x - minX) * sx + x1);
                int yp = (int) (h - (y - minY) * sy - y1);
                yp = yp - h1;
                g.fillRect(xp, yp, w1, h1);
            }
        }
    }

    // --------------------------------------------------------------------
    // legend
    // --------------------------------------------------------------------
    private void renderLegend(PlotDataXpm data, Graphics2D g, boolean pca) {
        int w1 = 0;
        if (pca) {
            w1 = (int) Math.ceil((x2 - x1) / data.numPointValues() * 0.7);
        } else {
            w1 = (int) Math.ceil((x2 - x1) / data.numPointValues() * 0.25);
        }
        int h1 = (int) Math.ceil((y2 - y1) / data.numPointValues());
        int ratio1 = (w1 < h1) ? w1 : h1;

        int fontSize = (int) (0.75 * ratio1);

        if (fontSize == 0) {
            fontSize = 1;
        } else if (fontSize > 12) {
            fontSize = 12;
        }
        Font font = new Font("Dialog", Font.PLAIN, fontSize);
        FontMetrics fm = g.getFontMetrics(font);

        for (int n = 0; n < data.numPointValues(); n++) {
            String legend = data.getPointValue(n).getLegend();

            int xp = (int) (x2 + 2 * fm.getHeight());
            int yp = (int) (h - y2 + n * fm.getHeight());

            g.setColor(data.getPointValue(n).getColor());
            g.fillRect(xp, yp, fm.getHeight(), fm.getHeight());

            g.setColor(Color.BLACK);
            g.drawRect(xp, yp, fm.getHeight(), fm.getHeight());

            xp = xp + (int) (1.3 * fm.getHeight());
            yp = yp + fm.getAscent();
            g.setFont(font);
            g.drawString(legend, xp, yp);
        }
    }

    // --------------------------------------------------------------------
    // picked value
    // --------------------------------------------------------------------
    private void renderPickedValue(PlotDataXpm data, Graphics2D g) {
        if (data.getPickIndexX() != null) {
            int fontSize1 = (int) (14 * ratio);
            Font font1 = new Font("Dialog", Font.PLAIN, fontSize1);
            FontMetrics fm1 = g.getFontMetrics(font1);
            int ascent1 = fm1.getAscent();

            float x = data.getXaxis(data.getPickIndexX());
            int xp = (int) ((x - minX) * sx + x1);
            int yp = (int) (h - y1 + 4.5 * ascent1);

            String xes = "x = ";
            g.setColor(Color.BLACK);
            g.setFont(font1);
            g.drawString(xes, xp, yp);

            xp = xp + fm1.stringWidth(xes);

            String xs = formatValue(x);
            g.drawString(xs, xp, yp);
        }
    }
}
