package jp.riken.viewer.gromacs;

import java.io.*;

public class GromacsGroReader extends GromacsMoleculeReader {

    private int positions;

    protected void readAsReference(BufferedReader reader) throws IOException {
        positions = 0;

        // first line
        String s = null;
        if ((s = reader.readLine()) == null) {
            return;
        }
        processTitle(s);

        // second line
        if ((s = reader.readLine()) == null) {
            return;
        }
        int natom = Integer.parseInt(s.trim());

        while ((s = reader.readLine()) != null) {
            if (positions == 0) {
                determinePositions(s);
                if (positions <= 0) {
                    return;
                }
            }
            processAtom(s);
            if (atomId == natom) {
                break;
            }
        }
    }

    private void processTitle(String s) {
        int index = s.indexOf("t=");
        if (index < 0) {
            title = s.trim();
        } else {
            title = s.substring(0, index).trim();
        }
    }

    private void determinePositions(String s) {
        int index1 = s.indexOf(".");
        if (index1 < 0) {
            return;
        }
        int index2 = s.indexOf(".", index1 + 1);
        if (index2 < 0) {
            return;
        }
        positions = index2 - index1;
    }

    private void processAtom(String s) {
        atomId++;
        String currResidueId = s.substring(0, 5);
        if (residueIdColumn == null || !residueIdColumn.equals(currResidueId)) {
            residueId++;
            residueIdColumn = currResidueId;
        }
        String residueName = s.substring(5, 10).trim().toUpperCase();
        residueName = fixResidueName(residueName);
        if (!ligand && residueName.equals(RESIDUE_LIGAND)) {
            builder.breakChain();
            ligand = true;
        }
        if (!solvent && residueName.equals(RESIDUE_SOLVENT)) {
            builder.breakChain();
            solvent = true;
        }
        String atomName = s.substring(10, 15).trim().toUpperCase();
        atomName = fixAtomName(atomName);
        atomRecord.setId("" + atomId);
        atomRecord.setAuthAsymId(" ");
        atomRecord.setSeqId("" + residueId);
        atomRecord.setAltId("");
        atomRecord.setCompId(residueName);
        atomRecord.setAtomId(atomName);

        int beg = 20;
        atomRecord.setCartnX("" + getCoord(s, beg));
        beg += positions;
        atomRecord.setCartnY("" + getCoord(s, beg));
        beg += positions;
        atomRecord.setCartnZ("" + getCoord(s, beg));

        atomRecord.setBFactor("0");
        atomRecord.setInsCode(' ');
        char c = atomName.charAt(0);
        if (c >= 'A' && c <= 'Z') {
            atomRecord.setTypeSymbol(atomName.substring(0, 2));
        } else {
            atomRecord.setTypeSymbol(atomName.substring(1, 2));
        }

        builder.addAtomRecord(atomRecord, false);
    }
    
    private String fixResidueName(String residueName) {
        if (residueName.length() < 3) {
            String name = "  " + residueName;
            return name.substring(name.length() - 3);
        }
        return residueName;
    }
    
    private String fixAtomName(String atomName) {
        String name = "   " + atomName + "  ";
        int i = 3;
        for (; i < 7; i++) {
            char ch = name.charAt(i);
            if (ch >= 'A' && ch <= 'Z') {
                break;
            }
        }
        if (i == 7) {
            name = "   " + atomName;
            return name.substring(name.length() - 4);
        } else if (i == 3 && atomName.length() == 4) {
            if (name.charAt(3) == 'H'
                    && name.charAt(6) >= '0' && name.charAt(6) <= '9') {
                return name.charAt(6) + name.substring(3, 6);
            } else {
                return atomName;
            }
        } else {
            return name.substring(i - 1, i + 3);
        }
    }

    private float getCoord(String s, int beg) {
        int end = beg + positions;
        return Float.parseFloat(s.substring(beg, end).trim()) * 10;
    }
}
