package jp.riken.viewer.gromacs;

import java.io.*;
import jp.riken.common.exception.*;
import org.pdbj.viewer.moleculebuilders.*;
import org.pdbj.viewer.molecules.*;

public abstract class GromacsMoleculeReader {

    public final String RESIDUE_LIGAND = "LIG";
    public final String RESIDUE_SOLVENT = "SOL";

    protected MoleculeFileBuilder builder;
    protected AtomRecord atomRecord;
    protected int frame;
    protected int atomId;
    protected int residueId;
    protected String residueIdColumn;
    protected String title;
    protected boolean ligand;
    protected boolean solvent;

    public MoleculeFileTraj readAsReference(File file, TrajectoryReaderOption option) {
        builder = new MoleculeFileBuilder();
        builder.setPDBml(false);
        builder.setFilePath(file.getAbsolutePath());
        builder.setFilter(option);
        init();

        try {
            BufferedReader reader = new BufferedReader(new FileReader(file));
            try {
                readAsReference(reader);
            } finally {
                try {
                    reader.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

        } catch (IOException e) {
            throw new CustomException(ErrorMessages.CANT_READ_FILE, e);
        } catch (Exception e) {
            throw new CustomException(ErrorMessages.INVALID_CONTENTS, e);
        }

        MoleculeFile moleculeFile = builder.getResult(true);
        moleculeFile.setMoleculeName(title);
        return new MoleculeFileTraj(moleculeFile);
    }

    protected void init() {
        atomRecord = new AtomRecord();
        atomRecord.setMdlId(AtomRecord.NO_MODEL_ID);
        frame = 0;
        atomId = 0;
        residueId = 0;
        residueIdColumn = null;
        title = null;
        ligand = false;
        solvent = false;
    }

    protected abstract void readAsReference(BufferedReader reader) throws IOException;
}
