package jp.riken.viewer.gromacs;

import java.io.*;

public class GromacsPdbReader extends GromacsMoleculeReader{

    protected void readAsReference(BufferedReader reader) throws IOException {
        String s = null;
        while ((s = reader.readLine()) != null) {
            // handle short header
            if (s.length() < 6) {
                s = addSpace(s, 6);
            }

            if (s.substring(0, 6).equals("TITLE")) {
                processTitle(s);
            } else if (s.substring(0, 6).equals("MODEL ")) {
                frame++;
                if (frame > 1) {
                    break;
                }
            } else if (s.substring(0, 6).equals("ATOM  ")) {
                processAtom(s);
            } else if (s.substring(0, 6).equals("TER   ")) {
                processTer(s);
            } else if (s.substring(0, 6).equals("ENDMDL")) {
                break;
            }
        }
    }

    private void processTitle(String s) {
        int index = s.indexOf("t=");
        if (index < 0) {
            title = s.substring(10).trim();
        } else {
            title = s.substring(10, index).trim();
        }
    }

    private void processAtom(String s) {
        atomId++;
        String currResidueId = s.substring(22, 26);
        if (residueIdColumn == null || !residueIdColumn.equals(currResidueId)) {
            residueId++;
            residueIdColumn = currResidueId;
        }
        String residueName = s.substring(17, 20).toUpperCase();
        if (!ligand && residueName.equals(RESIDUE_LIGAND)) {
            builder.breakChain();
            ligand = true;
        }
        if (!solvent && residueName.equals(RESIDUE_SOLVENT)) {
            builder.breakChain();
            solvent = true;
        }
        atomRecord.setId("" + atomId);
        atomRecord.setAuthAsymId(s.substring(21, 22));
        atomRecord.setSeqId("" + residueId);
        atomRecord.setAltId(s.substring(16, 17).trim());
        atomRecord.setCompId(residueName);
        atomRecord.setAtomId(s.substring(12, 16).toUpperCase());
        atomRecord.setCartnX(s.substring(30, 38).trim());
        atomRecord.setCartnY(s.substring(38, 46).trim());
        atomRecord.setCartnZ(s.substring(46, 54).trim());
        atomRecord.setBFactor(s.substring(60, 66).trim());
        atomRecord.setInsCode(s.charAt(26));
        char c = s.charAt(12);
        if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z')) {
            atomRecord.setTypeSymbol(s.substring(12, 14));
        } else {
            atomRecord.setTypeSymbol(s.substring(13, 14));
        }

        builder.addAtomRecord(atomRecord, false);
    }

    private void processTer(String s) {
        builder.breakChain();
    }

    private static String addSpace(String s, int column) {
        int len = s.length();
        if (len < column) {
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < column - len; i++) {
                sb.append(' ');
            }
            s += sb.toString();
        }
        return s;
    }
}
