package jp.riken.viewer.gui;

import java.io.File;
import javax.swing.filechooser.FileFilter;

public class GromacsFileType extends FileFilter {

    public static final GromacsFileType GRO = new GromacsFileType(0);
    public static final GromacsFileType PDB = new GromacsFileType(1);
    public static final GromacsFileType REF = new GromacsFileType(2);
    public static final GromacsFileType XTC = new GromacsFileType(3);
    public static final GromacsFileType XVG = new GromacsFileType(4);
    public static final GromacsFileType XPM = new GromacsFileType(5);
    public static final GromacsFileType PCA = new GromacsFileType(6);
    public static final GromacsFileType PNG = new GromacsFileType(7);
    public static final GromacsFileType JPG = new GromacsFileType(8);
    public static final GromacsFileType NDX = new GromacsFileType(9);

    private static final String[][] EXTENSIONS = {
        {"gro"},         // GRO
        {"pdb"},         // PDB
        {"gro", "pdb"},  // REF
        {"xtc"},         // XTC
        {"xvg"},         // XVG
        {"xpm"},         // XPM
        {"gro"},         // PCA
        {"png"},         // PNG
        {"jpg", "jpeg"}, // JPG
        {"ndx"}          // NDX
        };

    private int type;

    private GromacsFileType(int type) {
        this.type = type;
    }

    //--------------------------------------------------------------------
    // Implements FileFilter
    //--------------------------------------------------------------------
    public String getDescription() {
        if (EXTENSIONS[type].length == 0) {
            return "all files (*.*)";
        } else {
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < EXTENSIONS[type].length; i++) {
                if (i > 0) {
                    sb.append(", ");
                }
                sb.append("*." + EXTENSIONS[type][i]);
            }
            return sb.toString();
        }
    }

    public boolean accept(File file) {
        if (file.isDirectory() || EXTENSIONS[type].length == 0) {
            return true;
        }
        String name = file.getName();
        int index = name.lastIndexOf(".");
        if (index != -1) {
            String ext = name.substring(index + 1).toLowerCase();
            for (int i = 0; i < EXTENSIONS[type].length; i++) {
                if (ext.equals(EXTENSIONS[type][i])) {
                    return true;
                }
            }
        }
        return false;
    }
}
