package jp.riken.viewer.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import jp.riken.common.gui.*;

public class OpenHbondDialog extends DialogBase implements ActionListener,
        DocumentListener {

    private static final long serialVersionUID = 1L;

    private ApplicationFrame applicationFrame;
    private JTextField indexFileText;
    private JTextField mapFileText;
    private JButton indexFileBrowse;
    private JButton mapFileBrowse;
    private JButton buttonOK;
    private JButton buttonCancel;
    private boolean ok;

    public OpenHbondDialog(ApplicationFrame applicationFrame, String title,
            boolean modal) {
        super(applicationFrame, title, modal);
        this.applicationFrame = applicationFrame;
        setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
    }

    public void initialize() {
        ok = false;
    }

    public File getIndexFile() {
        return new File(indexFileText.getText());
    }

    public File getMapFile() {
        return new File(mapFileText.getText());
    }

    public boolean isOk() {
        return ok;
    }

    private void toggleOkButton() {
        if (buttonOK != null) {
            buttonOK.setEnabled(validateInput());
        }
    }

    private boolean validateInput() {
        if (!hasValue(indexFileText)) {
            return false;
        }
        if (!hasValue(mapFileText)) {
            return false;
        }
        return true;
    }

    // --------------------------------------------------------------------
    // ActionListener
    // --------------------------------------------------------------------
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == indexFileBrowse) {
            File file = applicationFrame.chooseFile(GromacsFileType.NDX, false);
            if (file != null) {
                indexFileText.setText(file.getAbsolutePath());
            }
        } else if (e.getSource() == mapFileBrowse) {
            File file = applicationFrame.chooseFile(GromacsFileType.XPM, false);
            if (file != null) {
                mapFileText.setText(file.getAbsolutePath());
            }
        } else if (e.getSource() == buttonOK) {
            ok = true;
            setVisible(false);
        } else if (e.getSource() == buttonCancel) {
            ok = false;
            setVisible(false);
        }
    }

    // --------------------------------------------------------------------
    // DocumentListener
    // --------------------------------------------------------------------
    public void changedUpdate(DocumentEvent e) {
    }

    public void insertUpdate(DocumentEvent e) {
        toggleOkButton();
    }

    public void removeUpdate(DocumentEvent e) {
        toggleOkButton();
    }

    // --------------------------------------------------------------------
    // Component
    // --------------------------------------------------------------------
    @Override
    protected void createComponent() {
        setSize(500, 220);
        setResizable(true);
        Container container = this.getContentPane();
        container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));

        Font font = new Font("SansSerif", Font.PLAIN, 12);
        indexFileText = createTextField(font, this);
        mapFileText = createTextField(font, this);
        indexFileBrowse = createButton("Browse...", font, this);
        mapFileBrowse = createButton("Browse...", font, this);
        buttonCancel = createButton("Cancel", font, this);
        buttonOK = createButton("  OK  ", font, this, 
                buttonCancel.getPreferredSize());
        buttonOK.setEnabled(false);

        Font font2 = new Font("SansSerif", Font.PLAIN, 12);
        JLabel label1 = createLabel("H-bond index file (.ndx): ", font2);
        JLabel label2 = createLabel("H-bond map file (.xpm): ", font2);

        JPanel panel = null;
        int mh1 = 20;
        int mh2 = 40;
        int mv1 = 20;

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(1, 20)));
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(mh1, 1)));
        panel.add(label1);
        panel.add(Box.createHorizontalGlue());
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(mh2, 1)));
        panel.add(indexFileText);
        panel.add(Box.createRigidArea(new Dimension(5, 1)));
        panel.add(indexFileBrowse);
        panel.add(Box.createRigidArea(new Dimension(20, 1)));
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(1, mv1)));
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(mh1, 1)));
        panel.add(label2);
        panel.add(Box.createHorizontalGlue());
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(mh2, 1)));
        panel.add(mapFileText);
        panel.add(Box.createRigidArea(new Dimension(5, 1)));
        panel.add(mapFileBrowse);
        panel.add(Box.createRigidArea(new Dimension(20, 1)));
        container.add(panel);

        container.add(Box.createVerticalGlue());

        panel = createPanel();
        panel.add(Box.createHorizontalGlue());
        panel.add(buttonOK);
        panel.add(Box.createRigidArea(new Dimension(5, 1)));
        panel.add(buttonCancel);
        panel.add(Box.createRigidArea(new Dimension(5, 1)));
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(1, 5)));
        container.add(panel);
    }
}
