package jp.riken.viewer.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import jp.riken.common.gui.*;

public class OpenPcaDialog extends DialogBase implements ActionListener,
        DocumentListener {

    private static final long serialVersionUID = 1L;

    private ApplicationFrame applicationFrame;
    private JTextField pcaFileText;
    private JTextField startText;
    private JTextField stepText;
    private JButton pcaFileBrowse;
    private JButton buttonOK;
    private JButton buttonCancel;
    private boolean ok;

    public OpenPcaDialog(ApplicationFrame applicationFrame, String title,
            boolean modal) {
        super(applicationFrame, title, modal);
        this.applicationFrame = applicationFrame;
        setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
    }

    public void initialize() {
        ok = false;
    }

    public File getPcaFile() {
        return new File(pcaFileText.getText());
    }

    public float getStart() {
        return Float.parseFloat(startText.getText());
    }

    public float getStep() {
        return Float.parseFloat(stepText.getText());
    }

    public boolean isOk() {
        return ok;
    }

    private void toggleOkButton() {
        if (buttonOK != null) {
            buttonOK.setEnabled(validateInput());
        }
    }

    private boolean validateInput() {
        if (!hasValue(pcaFileText)) {
            return false;
        }
        if (!isFloat(startText)) {
            return false;
        }
        float step = getAsFloat(stepText, 0);
        if (step <= 0) {
            return false;
        }
        return true;
    }

    // --------------------------------------------------------------------
    // ActionListener
    // --------------------------------------------------------------------
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == pcaFileBrowse) {
            File file = applicationFrame.chooseFile(GromacsFileType.PCA, false);
            if (file != null) {
                pcaFileText.setText(file.getAbsolutePath());
            }
        } else if (e.getSource() == buttonOK) {
            ok = true;
            setVisible(false);
        } else if (e.getSource() == buttonCancel) {
            ok = false;
            setVisible(false);
        }
    }

    // --------------------------------------------------------------------
    // DocumentListener
    // --------------------------------------------------------------------
    public void changedUpdate(DocumentEvent e) {
    }

    public void insertUpdate(DocumentEvent e) {
        toggleOkButton();
    }

    public void removeUpdate(DocumentEvent e) {
        toggleOkButton();
    }

    // --------------------------------------------------------------------
    // Component
    // --------------------------------------------------------------------
    @Override
    protected void createComponent() {
        setSize(500, 220);
        setResizable(true);
        Container container = this.getContentPane();
        container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));

        Font font = new Font("SansSerif", Font.PLAIN, 12);
        pcaFileText = createTextField(font, this);
        startText = createTextField(font, this, 70);
        stepText = createTextField(font, this, 70);
        pcaFileBrowse = createButton("Browse...", font, this);
        buttonCancel = createButton("Cancel", font, this);
        buttonOK = createButton("  OK  ", font, this,
                buttonCancel.getPreferredSize());
        buttonOK.setEnabled(false);

        Font font2 = new Font("SansSerif", Font.PLAIN, 12);
        JLabel label1 = createLabel("PCA file (.gro): ", font2);
        JLabel label2 = createLabel("Time: ", font2);
        JLabel label21 = createLabel("start (ps) ", font2);
        JLabel label22 = createLabel("step (ps) ", font2);

        JPanel panel = null;
        int mh1 = 20;
        int mh2 = 40;
        int mv1 = 20;

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(1, 20)));
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(mh1, 1)));
        panel.add(label1);
        panel.add(Box.createHorizontalGlue());
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(mh2, 1)));
        panel.add(pcaFileText);
        panel.add(Box.createRigidArea(new Dimension(5, 1)));
        panel.add(pcaFileBrowse);
        panel.add(Box.createRigidArea(new Dimension(20, 1)));
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(1, mv1)));
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(mh1, 1)));
        panel.add(label2);
        panel.add(Box.createHorizontalGlue());
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(mh2, 1)));
        panel.add(label21);
        panel.add(startText);
        panel.add(Box.createRigidArea(new Dimension(20, 1)));
        panel.add(label22);
        panel.add(stepText);
        panel.add(Box.createHorizontalGlue());
        container.add(panel);

        container.add(Box.createVerticalGlue());

        panel = createPanel();
        panel.add(Box.createHorizontalGlue());
        panel.add(buttonOK);
        panel.add(Box.createRigidArea(new Dimension(5, 1)));
        panel.add(buttonCancel);
        panel.add(Box.createRigidArea(new Dimension(10, 1)));
        container.add(panel);

        panel = createPanel();
        panel.add(Box.createRigidArea(new Dimension(1, 10)));
        container.add(panel);

        startText.setText("0");
        stepText.setText("10");
    }
}
